/****************************************************************
 Licensed Materials - Property of IBM and/or HCL 
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/

package com.hcl.test.rm.model;

import java.util.Collections;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.Map;

import io.swagger.annotations.ApiModelProperty;

public class RMModelBase {

	/** Those properties allow to make public some more data to the consumers/queriers/... */
	@ApiModelProperty(notes = "Properties are used to provide more specific information about the entity; it's a map of key/value pairs where key is a string and value a string, number or boolean; possible key and values are described in the field providedProperties of RMSourceInfo")
	private Map<String, Object> properties;
	
	// This one has no getter/setter, so it is not serialized, which is intentional as it aims to be useful to Data Collectors internally
	private Map<String, Object> arbitraries;

	/** public only for serialization, returned map cannot be modified, use setProperty/getProperty */
	public Map<String, Object> getProperties() {
		return properties != null ? Collections.unmodifiableMap(properties) : null;
	}

	/**
	 * Adds a public property to this observable; such properties are useful when resolving end-user selection queries
	 * @param key must not be empty neither null
	 * @param value only accepted types are primitive types, their wrappers and String
	 */
	void setProperty(String key, Object value) {
		if (value != null) {
			if (properties == null) {
				properties = new LinkedHashMap<>();
			}
			properties.put(key, value);
		} else if (properties != null) {
			properties.remove(key);
		}
	}
	
	public boolean hasProperty(String key) {
		return properties != null && properties.containsKey(key);
	}
	
	public Object getProperty(String key) {
		return properties != null ? properties.get(key) : null;
	}

	/**
	 * An arbitrary allows to associate internal arbitrary data, helpful to cache more building/retrieving information.<br/>
	 * Arbitraries are not serialized and not visible to the consumers/queriers/end-users. They remain in memory with the observable tree.
	 */
	void setArbitrary(String key, Object value) {
		if (value != null) {
			if (arbitraries == null) {
				arbitraries = new Hashtable<>();
			}
			arbitraries.put(key, value);
		} else if (arbitraries != null) {
			arbitraries.remove(key);
		}
	}
	
	public boolean hasArbitrary(String key) {
		return arbitraries != null && arbitraries.containsKey(key);
	}
	
	public Object getArbitrary(String key) {
		return arbitraries != null ? arbitraries.get(key) : null;
	}

}
