/****************************************************************
 Licensed Materials - Property of IBM and/or HCL 
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/

package com.hcl.test.rm.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY)
@JsonSubTypes({
	@Type(value = RMObservableContainer.class,	name = "RMObservableContainer"),
	@Type(value = RMObservableData.class,		name = "RMObservableData")
})
@JsonInclude(Include.NON_NULL)
// TODO: not supported yet (see https://github.com/springfox/springfox/issues/425)
@ApiModel(/*subTypes = { RMObservableContainer.class, RMObservableData.class }, discriminator = "@type"*/
		  description = "Shared base for RMObservableContainer and RMObservableData; concrete type is given in the '@type' property")
public abstract class RMObservable extends RMModelBase {

	private static final String LAZY_MARKER = "<lazy>";

	/** Unique id of the observable, built from parent and local name, transformed to an HTML valid name/id attributes value */
	@ApiModelProperty(notes = "String uniquely identifying this entity in the observable tree")
	private String id;
	
	/** Internal name of the observable, meaningful to the data collector, used to build the id */
	@ApiModelProperty(hidden = true)
	private String name;
	
	/** Short description of the observable */
	@ApiModelProperty(notes = "User facing short description of the observable")
	private String description;
	
	/** Long optional explanation of the observable, null or not set if not available and set to {@link #LAZY_MARKER} if it can be lazily loaded */
	@ApiModelProperty(notes = "User facing longer explanation about the observable; optional")
	private String explanation;
	
	void setId(String id) {
		this.id = id;
	}
	
	/**
	 * Made of the internal concatenation of the traversed nodes from root to ensure uniqueness.<br/>
	 * Node (container or leaf) id allows observable data selections, lazy load and selective refresh of container's children.
	 * @return unique id of this node
	 */
	public String getId() {
		return id;
	}
	
	void setName(String name) {
		this.name = name;
	}
	
	public String getName() {
		return name;
	}
	
	void setDescription(String description) {
		this.description = description;
	}
	
	public String getDescription() {
		return description;
	}
	
	void setLazyExplanation() {
		this.explanation = LAZY_MARKER;
	}

	@JsonIgnore
	public boolean isLazyExplanation() {
		return LAZY_MARKER.equals(this.explanation);
	}
	
	void setExplanation(String explanation) {
		this.explanation = explanation;
	}
	
	public String getExplanation() {
		return explanation;
	}
	
}
