/****************************************************************
 Licensed Materials - Property of IBM and/or HCL 
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/

package com.hcl.test.rm.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

@ApiModel(description = "Group of observable datas or other containers in the observable tree")
public class RMObservableContainer extends RMObservable {

	@JsonInclude(JsonInclude.Include.NON_DEFAULT)
	@ApiModelProperty(hidden = true, notes = "In case children are lazily loaded, lazy is true; GET it with /source-observables/{sourceId}/{observableId} endpoint")
	private boolean lazy;
	
	@JsonInclude(JsonInclude.Include.NON_DEFAULT)
	@ApiModelProperty(hidden = true, notes = "In case children list may change at any time, refreshable is true; GET /source-observables/{sourceId}/{observableId} endpoint to get fresh list")
	private boolean refreshable;
	
	@ApiModelProperty(notes = "Children can be made of any of the classes extending RMObservable, i.e. RMObservableData and RMObservableContainer")
	private List<RMObservable> children;
	
	/**
	 * When set as lazy, a container can be loaded at any time to populate its children.<br/>
	 * While lazy and refreshable can be set together, it is not mandatory as lazy rather means that the data is longer to load and doesn't make
	 * sense at first to any end-user. Usually, the actual load of an observable container's children will occur when the end-user explicitely
	 * requests it to load.
	 */
	public void setLazy(boolean lazy) {
		this.lazy = lazy;
	}
	
	public boolean isLazy() {
		return lazy;
	}
	
	/**
	 * When set as refreshable, a container can be refreshed again and again at any time to reflect its actual children.
	 */
	public void setRefreshable(boolean refreshable) {
		this.refreshable = refreshable;
	}
	
	public boolean isRefreshable() {
		return refreshable;
	}
	
	void addChild(RMObservable child) {
		if (children == null) {
			children = new ArrayList<>();
		}
		children.add(child);
	}
	
	void addAllChildren(Collection<? extends RMObservable> allChildren) {
		if (children == null) {
			children = new ArrayList<>();
		}
		children.addAll(allChildren);
	}
	
	void clearChildren() {
		children = null;
	}
	
	public List<RMObservable> getChildren() {
		return children;
	}

}
