/****************************************************************
 Licensed Materials - Property of IBM and/or HCL 
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/

package com.hcl.test.rm.model;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

@ApiModel(description = "Defines an observable data, i.e. a leaf in the observable tree")
public class RMObservableData extends RMObservable {

	@ApiModelProperty("User facing unit of the data; optional")
	private String unit;
	
	public enum SpecialHandling {
		/**
		 * To be handled as a percentage: y axis is always ticked from 0 to 100
		 */
		PERCENT,
		/**
		 * Delta computed from previously retrieved value: always ignore first value in the set if it is NaN, meaning no previous value
		 */
		DELTA,
		// TODO: there may be some more to add: UNCAPPED_PERCENT (can be more than 100%), ...
	}
	
	@ApiModelProperty(hidden = true)
	private SpecialHandling specialHandling;
	
	void setUnit(String unit) {
		this.unit = unit;
	}
	
	public String getUnit() {
		return unit;
	}
	
	public SpecialHandling getSpecialHandling() {
		return specialHandling;
	}
	
	/**
	 * Setting a special handling will give more mean to the data for consumers to inerpret, see {@link SpecialHandling} possible values
	 * @param specialHandling
	 */
	void setSpecialHandling(SpecialHandling specialHandling) {
		this.specialHandling = specialHandling;
	}

}
