/****************************************************************
 Licensed Materials - Property of IBM and/or HCL 
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/

package com.hcl.test.rm.model;

import java.util.Map;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import io.swagger.annotations.ApiModelProperty;

public class RMSourceInfo extends RMModelBase {
	
	@ApiModelProperty(notes = "The id of the Data Collector that this source is made of")
	private String dataCollectorId;
	
	@ApiModelProperty(notes = "A string uniquely identifying this source in the Service")
	private String sourceId;
	
	@ApiModelProperty(hidden = true)
	private long parametersVersion;

	@ApiModelProperty(notes = "The provided title of this source; may have been modified by a team member in the Service's web page")
	private String title;

	@ApiModelProperty(notes = "The definitions of observables start here; only the observable containers loaded during configuration have their children populated")
	private RMObservableContainer observableRoot;
	
	@ApiModelProperty(notes = "Some explanations about the properties provided by the data collector for this source")
	private Map<String, String> providedProperties;
	
	@ApiModelProperty(notes = "Timestamp represented as number of milliseconds since Unix Epoch")
	private long addedOn, lastObservedOn;
	
	@ApiModelProperty(hidden = true)
	private boolean partlyLoadable;
	
	@JsonInclude(Include.NON_NULL)
	@ApiModelProperty(notes = "An healthy source has no configuration exception")
	private RMConfigurationException configurationException;
	
	@ApiModelProperty(notes = "An array of of the ids of the selected observable datas")
	private Set<String> selectedObservableDatas;

	void setDataCollectorId(String dataCollectorId) {
		this.dataCollectorId = dataCollectorId;
	}
	
	public String getDataCollectorId() {
		return dataCollectorId;
	}
	
	void setSourceId(String sourceId) {
		this.sourceId = sourceId;
	}
	
	public String getSourceId() {
		return sourceId;
	}
	
	void setParametersVersion(long parametersVersion) {
		this.parametersVersion = parametersVersion;
	}
	
	public long getParametersVersion() {
		return parametersVersion;
	}
	
	void setTitle(String title) {
		this.title = title;
	}
	
	public String getTitle() {
		return title;
	}

	void setObservableRoot(RMObservableContainer observableRoot) {
		this.observableRoot = observableRoot;
	}
	
	public RMObservableContainer getObservableRoot() {
		return observableRoot;
	}
	
	void setProvidedProperties(Map<String, String> providedProperties) {
		this.providedProperties = providedProperties;
	}
	
	/**
	 * Properties can be associated to {@link RMObservable} and publicly available for reading by consumers.<br>
	 * They allow to provide more criteria when selecting observables with a query for instance.
	 * @return a map where key is the property name and the value is english (TODO: or translated) explanation of what it means and what it is set for
	 * @see {@link RMObservable#setProperty(String, Object)} and {@link RMObservable#getProperty(String)} 
	 */
	public Map<String, String> getProvidedProperties() {
		return providedProperties;
	}

	public void setAddedOn(long addedOn) {
		this.addedOn = addedOn;
	}
	
	public long getAddedOn() {
		return addedOn;
	}
	
	public void setLastObservedOn(long lastObservedOn) {
		this.lastObservedOn = lastObservedOn;
	}
	
	public long getLastObservedOn() {
		return lastObservedOn;
	}
	
	void setConfigurationException(RMConfigurationException configurationException) {
		this.configurationException = configurationException;
	}
	
	public RMConfigurationException getConfigurationException() {
		return configurationException;
	}
	
	/**
	 * Internally called, rather use {@link #getSelectedObservableDatas()} when building a fresh set of observations for this set of observable datas
	 * @param selectedObservableDatas
	 */
	public void setSelectedObservableDatas(Set<String> selectedObservableDatas) {
		this.selectedObservableDatas = selectedObservableDatas;
	}
	
	/**
	 * Useful when building a fresh set of observations: only observable data ids contained in this set are to be retrieved
	 * @return the current end-user selection
	 */
	public Set<String> getSelectedObservableDatas() {
		return selectedObservableDatas;
	}

	/**
	 * No need to be called by the Data Collectors: it's updated when the Source is created, looking at the implemented interfaces
	 * @param isPartlyLoadable
	 */
	public void setPartlyLoadable(boolean isPartlyLoadable) {
		this.partlyLoadable = isPartlyLoadable;
	}

	public boolean isPartlyLoadable() {
		return partlyLoadable;
	}
	
}
