/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "jquery", "LayoutParticipant", "CascadableLayoutManager", "DialogsManager", "SourceLayout", "Messages", "Nav_template", "DataCollectors_template", "bootstrap", "moment-with-locales"], function (require, exports, $, LayoutParticipant_1, CascadableLayoutManager_1, DialogsManager_1, SourceLayout_1, Messages_1, Nav_template_1, DataCollectors_template_1) {
    "use strict";
    exports.__esModule = true;
    var DataCollectorsAdapter = /** @class */ (function () {
        function DataCollectorsAdapter(dataCollectors) {
            this.dataCollectors = dataCollectors;
        }
        DataCollectorsAdapter.prototype.forId = function (id) {
            if (!this.dataCollectors)
                return null;
            for (var _i = 0, _a = this.dataCollectors; _i < _a.length; _i++) {
                var dc = _a[_i];
                if (dc.dataCollectorId === id)
                    return dc;
            }
            return null;
        };
        DataCollectorsAdapter.prototype.isInUse = function (dataCollector) {
            return dataCollector && dataCollector.properties && dataCollector.properties.isInUse;
        };
        DataCollectorsAdapter.prototype.canCreateOrModify = function (dataCollector) {
            return dataCollector && dataCollector.properties && dataCollector.properties.canCreateOrModify;
        };
        DataCollectorsAdapter.prototype.hasInUseDataCollectors = function () {
            var ret = false;
            if (!this.dataCollectors)
                return false;
            for (var _i = 0, _a = this.dataCollectors; _i < _a.length; _i++) {
                var dataCollector = _a[_i];
                if (this.isInUse(dataCollector)) {
                    ret = true;
                    break;
                }
            }
            ;
            return ret;
        };
        DataCollectorsAdapter.prototype.hasSingleInUseDataCollector = function () {
            var count = 0;
            if (!this.dataCollectors)
                return false; //by choice
            for (var _i = 0, _a = this.dataCollectors; _i < _a.length; _i++) {
                var dataCollector = _a[_i];
                if (this.isInUse(dataCollector)) {
                    count++;
                    if (count > 1)
                        break;
                }
            }
            ;
            return count == 1;
        };
        DataCollectorsAdapter.prototype.inUseDataCollectors = function () {
            var _this = this;
            var ret = [];
            this.dataCollectors.forEach(function (dataCollector) {
                if (_this.isInUse(dataCollector)) {
                    ret.push(dataCollector);
                }
            });
            return ret;
        };
        DataCollectorsAdapter.prototype.otherDataCollectors = function () {
            var _this = this;
            var ret = [];
            this.dataCollectors.forEach(function (dataCollector) {
                if (!_this.isInUse(dataCollector)) {
                    ret.push(dataCollector);
                }
            });
            return ret;
        };
        DataCollectorsAdapter.prototype.hasBoth = function () {
            var hasInUse = false;
            var hasOthers = false;
            for (var _i = 0, _a = this.dataCollectors; _i < _a.length; _i++) {
                var dataCollector = _a[_i];
                if (this.isInUse(dataCollector)) {
                    hasInUse = true;
                    if (hasOthers) {
                        break;
                    }
                }
                else {
                    hasOthers = true;
                    if (hasInUse) {
                        break;
                    }
                }
            }
            return hasInUse && hasOthers;
        };
        return DataCollectorsAdapter;
    }());
    var DataCollectorsLayout = /** @class */ (function (_super) {
        __extends(DataCollectorsLayout, _super);
        function DataCollectorsLayout() {
            var _this = _super.call(this, 'data-collectors', ['create-source', 'collapsible-sources-container', 'sources-container', 'extended-layout', 'responsive-layout', 'help-about-agent'], ['DATA_COLLECTORS_LIST_CHANGED', 'SOURCE_ADDED']) || this;
            _this.dialogsManager = new DialogsManager_1.DialogsManager();
            _this.sourcesLayoutManager = new CascadableLayoutManager_1.CascadableLayoutManager();
            _this.isExtendedLayout = false; //responsive by default
            setInterval(function () {
                _this.updateMoments();
            }, 10000); // 10 seconds is fine
            var layout = localStorage.getItem('dataCollectorsExtendedLayout');
            if ("true" === layout) {
                _this.isExtendedLayout = true;
            }
            else if ("false" === layout) {
                _this.isExtendedLayout = false;
            }
            return _this;
        }
        DataCollectorsLayout.prototype.refresh = function () {
            var _this = this;
            $.ajax({
                url: this.url,
                dataType: 'json',
                headers: {
                    'Cache-Control': 'max-age=0'
                },
                success: function (dataCollectors) {
                    _this.display(dataCollectors);
                },
                error: function (xhr, textStatus, errorThrown) {
                    $('[data-rm-role="data-collector-parent"]').trigger('server-error', [xhr, textStatus, errorThrown]);
                }
            });
            // No need to call sourcesLayoutManager.refresh() as it will cascade
        };
        DataCollectorsLayout.prototype.display = function (dataCollectors) {
            var data = { dataCollectors: dataCollectors };
            data['data-collectors-adapter'] = this.dataCollectorsAdapter = new DataCollectorsAdapter(dataCollectors);
            data['data-collectors-layout'] = this;
            // Update the menu:
            $('[data-rm-role="menu-add"]').replaceWith(Nav_template_1.addMenu(data));
            // See what to do with the content
            if (!this.dataCollectors) {
                // First time
                $('body').append(DataCollectors_template_1.dataCollectorsLayout(data)); // This will setup the placeholders for the data collectors
                if (this.dataCollectorsAdapter.hasInUseDataCollectors()) {
                    for (var _i = 0, dataCollectors_1 = dataCollectors; _i < dataCollectors_1.length; _i++) {
                        var dataCollector = dataCollectors_1[_i];
                        if (this.dataCollectorsAdapter.isInUse(dataCollector)) {
                            this.loadSources(dataCollector.dataCollectorId);
                        }
                    }
                }
            }
            else {
                // Not first time, let's be careful on what has changed
                if (!this.dataCollectorsAdapter.hasInUseDataCollectors()) {
                    // Back to first time user: none in use
                    $('[data-rm-role="data-collectors-container"]').replaceWith(DataCollectors_template_1.dataCollectorsLayout(data));
                }
                else {
                    $('[data-rm-role="no-data-collectors-tip-message"]').remove(); //remove tip message if exists
                    for (var _a = 0, dataCollectors_2 = dataCollectors; _a < dataCollectors_2.length; _a++) {
                        var dataCollector = dataCollectors_2[_a];
                        var existing = this.findDataCollector(dataCollector.dataCollectorId);
                        var wasInUse = this.dataCollectorsAdapter.isInUse(existing);
                        var isInUse = this.dataCollectorsAdapter.isInUse(dataCollector);
                        if (isInUse) {
                            dataCollector['data-collectors-adapter'] = this.dataCollectorsAdapter;
                            dataCollector['data-collectors-layout'] = this;
                            if (!wasInUse) {
                                // Let's remove its not-in-used container (if any: calling remove() on an empty JQuery set won't do anything)
                                $('[data-rm-role="data-collector-area"][data-rm-data-collector-id="' + dataCollector.dataCollectorId + '"]').remove();
                                // It's a new one, let's add it at the end
                                $('[data-rm-role="data-collector-parent"]').append(DataCollectors_template_1.dataCollectorLayout(dataCollector));
                            }
                            else {
                                // Update header only, sources will be refreshed just after
                                $('[data-rm-role="data-collector-header"][data-rm-data-collector-id="' + dataCollector.dataCollectorId + '"]').replaceWith(DataCollectors_template_1.dataCollectorHeader(dataCollector));
                            }
                            this.loadSources(dataCollector.dataCollectorId);
                        }
                        else {
                            // No longer needed here
                            $('[data-rm-role="data-collector-area"][data-rm-data-collector-id="' + dataCollector.dataCollectorId + '"]').remove();
                        }
                    }
                    this.updateLayout();
                }
            }
            this.dataCollectors = dataCollectors;
        };
        DataCollectorsLayout.prototype.loadSources = function (dataCollectorId) {
            var _this = this;
            $.ajax({
                url: 'sources?dataCollectorId=' + dataCollectorId,
                dataType: 'json',
                headers: {
                    'Cache-Control': 'max-age=0'
                },
                success: function (sourceInfos) {
                    _this.createAndDisplaySources(sourceInfos);
                },
                error: function (xhr, textStatus, errorThrown) {
                    $('[data-rm-role="data-collector-area"][data-rm-data-collector-id="' + dataCollectorId + '"]').trigger('server-error', [xhr, textStatus, errorThrown]);
                }
            });
        };
        DataCollectorsLayout.prototype.createAndDisplaySources = function (sourceInfos) {
            for (var _i = 0, sourceInfos_1 = sourceInfos; _i < sourceInfos_1.length; _i++) {
                var sourceInfo = sourceInfos_1[_i];
                var $existing = $('[data-rm-role="source-container"][data-rm-source-id="' + sourceInfo.sourceId + '"]');
                if ($existing.empty()) {
                    var sourceLayout = new SourceLayout_1.SourceLayout(this.dialogsManager, sourceInfo, this.sourcesLayoutManager);
                    sourceLayout.display(sourceInfo);
                    if (this.newlyCreatedSourceId && sourceInfo.sourceId === this.newlyCreatedSourceId) {
                        this.newlyCreatedSourceId = null;
                        this.dialogsManager.configureSourceDialog(sourceInfo, function () {
                            // KO: chart failed this.loadSources(dataCollectorId);
                            sourceLayout.refresh();
                        });
                    }
                } // else nothing to do here
            }
        };
        DataCollectorsLayout.prototype.findDataCollector = function (dataCollectorId) {
            for (var _i = 0, _a = this.dataCollectors; _i < _a.length; _i++) {
                var dataCollector = _a[_i];
                if (dataCollector.dataCollectorId === dataCollectorId) {
                    return dataCollector;
                }
            }
            return null;
        };
        DataCollectorsLayout.prototype.withUrl = function (url) {
            return _super.prototype.withUrl.call(this, url) || this.sourcesLayoutManager.withUrl(url);
        };
        DataCollectorsLayout.prototype.withRole = function (role) {
            return _super.prototype.withRole.call(this, role) || this.sourcesLayoutManager.withRole(role);
        };
        DataCollectorsLayout.prototype.withServerEvent = function (serverEvent) {
            return _super.prototype.withServerEvent.call(this, serverEvent) || this.sourcesLayoutManager.withServerEvent(serverEvent);
        };
        DataCollectorsLayout.prototype.decorateEventData = function (event) {
            var $dataCollectorContainer = event.data.$target.closest('[data-rm-data-collector-id][data-rm-data-collector-id!=""]');
            var dataCollectorId = $dataCollectorContainer.data('rmDataCollectorId');
            if (dataCollectorId) {
                event.data.$dataCollectorLayout = event.data.$target.closest('[data-rm-role="data-collector-area"]');
                ;
                event.data.dataCollectorId = dataCollectorId;
            }
            var $sourceLayout = event.data.$target.closest('[data-rm-source-id][data-rm-source-id!=""]');
            var sourceId = $sourceLayout.data('rmSourceId');
            if (sourceId) {
                event.data.$sourceLayout = $sourceLayout;
                event.data.sourceId = sourceId;
            }
            //console.log('decorated event: ' + event.type + ' role:' + event.data.role + ' dataCollectorId:' + event.data.dataCollectorId + ' sourceId:' + event.data.sourceId);
        };
        DataCollectorsLayout.prototype.onEvent = function (event) {
            var _this = this;
            switch (event.type) {
                case 'source-container-updated':
                    this.updateTitle(event.data.$dataCollectorLayout);
                    this.updateCounts(event.data.$dataCollectorLayout);
                    return false;
                case 'data-collector-updated':
                    this.refresh();
                    return false;
                default:
                    switch (event.data.role) {
                        case 'create-source': {
                            this.dialogsManager.createSourceDialog(event.data.dataCollectorId, function (request) {
                                var location = request ? request.getResponseHeader('Location') : '';
                                var index = location ? location.lastIndexOf('/') : -1;
                                if (index > 0) {
                                    _this.newlyCreatedSourceId = location.substring(index + 1);
                                }
                                else {
                                    _this.newlyCreatedSourceId = null; //no configure dialog automatically opened
                                }
                                _this.refresh();
                            });
                            return false;
                        }
                        case 'extended-layout':
                            this.setExtendedLayout(true);
                            break;
                        case 'responsive-layout':
                            this.setExtendedLayout(false);
                            break;
                        case 'help-about-agent':
                            this.dialogsManager.displayAboutAgentDialog();
                            return false;
                    }
                    // Pass through to sources
                    return this.sourcesLayoutManager.onEvent(event);
            }
        };
        DataCollectorsLayout.prototype.onServerEvent = function (event) {
            switch (event.operation) {
                case 'DATA_COLLECTORS_LIST_CHANGED':
                case 'SOURCE_ADDED':
                    this.refresh();
                    return false;
            }
            return this.sourcesLayoutManager.onServerEvent(event);
        };
        DataCollectorsLayout.prototype.setExtendedLayout = function (ext) {
            if (this.isExtendedLayout == ext)
                return;
            this.isExtendedLayout = ext;
            localStorage.setItem('dataCollectorsExtendedLayout', ext ? "true" : "false");
            // this.refresh();  //note: refresh is too heavy, ie it collapse all sources. 
            this.updateLayout();
            var $iconsExtended = $('[data-rm-layout-icon="extended"]');
            var $iconsResponsive = $('[data-rm-layout-icon="responsive"]');
            //remove check for all
            $iconsExtended.removeClass('glyphicon-ok rm-glyphicon-none');
            $iconsResponsive.removeClass('glyphicon-ok rm-glyphicon-none');
            if (this.isExtendedLayout) {
                $iconsExtended.addClass('glyphicon-ok');
                $iconsResponsive.addClass('rm-glyphicon-none');
            }
            else {
                $iconsExtended.addClass('rm-glyphicon-none');
                $iconsResponsive.addClass('glyphicon-ok');
            }
        };
        DataCollectorsLayout.prototype.updateLayout = function () {
            var _this = this;
            var $areas = $('[data-rm-role="data-collector-area"]');
            $areas.removeClass();
            $areas.each(function (index, element) {
                var $area = $(element);
                var dataCollectorId = $area.attr('data-rm-data-collector-id');
                var dataCollector = _this.dataCollectorsAdapter.forId(dataCollectorId);
                $area.addClass(_this.getDataCollectorAreaLayoutClasses(dataCollector));
            });
        };
        DataCollectorsLayout.prototype.getDataCollectorAreaLayoutClasses = function (dataCollector) {
            var isInUse = this.dataCollectorsAdapter.isInUse(dataCollector);
            if (isInUse) {
                var single = this.dataCollectorsAdapter.hasSingleInUseDataCollector();
                var extendedLayout = this.isExtendedLayout;
                if (extendedLayout || single) {
                    return 'col-xs-12 col-sm-12 col-md-12 col-lg-12'; //full width
                }
                else {
                    return 'col-xs-12 col-sm-12 col-md-6 col-lg-6'; //width depends on media
                }
            }
            else {
                return 'col-xs-12 col-sm-6 col-md-4 col-lg-3'; //width depends on media
            }
        };
        DataCollectorsLayout.prototype.updateTitle = function ($dataCollectorLayout) {
            var count = $dataCollectorLayout.find('[data-rm-role="source-container"]').length;
            if (count > 1) {
                var $candidates = $dataCollectorLayout.find('[data-rm-role="plural-candidate"]');
                $candidates.each(function (index, element) {
                    var $element = $(element);
                    $element.text($element.attr('data-role-plural-text'));
                });
            }
        };
        DataCollectorsLayout.prototype.updateCounts = function ($dataCollectorLayout) {
            this.updateCount($dataCollectorLayout, ['success', 'warning', 'error']);
            this.updateMoments();
        };
        DataCollectorsLayout.prototype.updateCount = function ($dataCollectorLayout, countTypes) {
            for (var _i = 0, countTypes_1 = countTypes; _i < countTypes_1.length; _i++) {
                var countType = countTypes_1[_i];
                var count = $dataCollectorLayout.find('[data-rm-role="data-collector-countable-' + countType + '"]').length;
                var badge = $dataCollectorLayout.find('[data-rm-role="data-collector-' + countType + '-count-placeholder"]');
                if (count > 0) {
                    badge.text(count);
                }
                else {
                    badge.text('');
                }
                var key = 'DC_' + countType.toUpperCase() + '_TITLE';
                var title = Messages_1.Messages.GetPlural(key, count);
                badge.prop('title', title);
            }
        };
        DataCollectorsLayout.prototype.updateMoments = function () {
            $('[data-rm-update="moment-update"]').each(function (index, element) {
                var $element = $(element);
                var serviceClockSkew = (typeof window['serviceClockSkew'] === 'number' ? window['serviceClockSkew'] : 0);
                var time = Math.min(parseInt($element.attr('data-rm-time-value')) + serviceClockSkew, new Date().getTime());
                var fromNow = moment(time).fromNow();
                var message = $element.attr('data-rm-message');
                $element.text(message.replace(/\{0\}/, fromNow));
            });
        };
        return DataCollectorsLayout;
    }(LayoutParticipant_1.LayoutParticipant));
    exports.DataCollectorsLayout = DataCollectorsLayout;
});
