/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "jquery", "FormHallBuilder", "TreeManager", "DataCollectors_template", "Nav_template", "bootstrap"], function (require, exports, $, FormHallBuilder_1, TreeManager_1, DataCollectors_template_1, Nav_template_1) {
    "use strict";
    exports.__esModule = true;
    var ObservablesTreeAdapter = /** @class */ (function (_super) {
        __extends(ObservablesTreeAdapter, _super);
        function ObservablesTreeAdapter(lazyUrlStart, $element) {
            var _this = _super.call(this) || this;
            _this.lazyUrlStart = lazyUrlStart;
            _this.$element = $element;
            _this.withCheckboxes = true;
            return _this;
        }
        ObservablesTreeAdapter.prototype.getId = function (item) { return item.id; };
        ObservablesTreeAdapter.prototype.getText = function (item) { return item.description; };
        ObservablesTreeAdapter.prototype.isContainer = function (item) { return item['@type'] === 'RMObservableContainer'; };
        ObservablesTreeAdapter.prototype.isExpanded = function (item) { return this.hasChildren(item); };
        ObservablesTreeAdapter.prototype.isRefreshable = function (item) { return item.refreshable && this.hasChildren(item); };
        ObservablesTreeAdapter.prototype.hasChildren = function (item) { return this.isContainer(item) && item.children && item.children.length > 0; };
        ObservablesTreeAdapter.prototype.getChildren = function (item) { return item.children; };
        ObservablesTreeAdapter.prototype.getExplanation = function (item) { return item.explanation; };
        ObservablesTreeAdapter.prototype.isLazyExplanation = function (item) { return item.explanation === '<lazy>'; };
        ObservablesTreeAdapter.prototype.loadItem = function (id, complete) {
            var _this = this;
            $.ajax({
                url: this.lazyUrlStart + id,
                dataType: 'json',
                headers: {
                    'Cache-Control': 'max-age=0'
                },
                success: function (data) {
                    complete(data);
                },
                error: function (xhr, textStatus, errorThrown) {
                    setTimeout(function () {
                        _this.$element.trigger('server-error', [xhr, textStatus, errorThrown]);
                    });
                    complete({ 'server-error': true });
                }
            });
        };
        return ObservablesTreeAdapter;
    }(TreeManager_1.TreeAdapter));
    var ConfigureDialogManager = /** @class */ (function () {
        function ConfigureDialogManager($modal, manager, onSaveSuccess) {
            var _this = this;
            this.$modal = $modal;
            this.manager = manager;
            this.onSaveSuccess = onSaveSuccess;
            this.dirty = false;
            this.$form = $modal.find('form');
            this.$form.ajaxForm(function () {
                $modal.modal('hide');
                onSaveSuccess();
            }).change(function () {
                _this.setDirty(true);
            }).keypress(function () {
                _this.setDirty(true);
            }).on('reset', function () {
                // Let the reset first do its job, then let's update the parent checkboxes
                setTimeout(function () {
                    manager.onReset();
                    _this.setDirty(false);
                });
            });
            this.disableSubmitReset(true);
        }
        ConfigureDialogManager.prototype.disableSubmitReset = function (disabled) {
            this.$form.find('button[type="submit"]').prop('disabled', disabled);
            this.$form.find('button[type="reset"]').prop('disabled', disabled);
        };
        ConfigureDialogManager.prototype.setDirty = function (dirty) {
            if (!this.dirty && dirty) {
                this.disableSubmitReset(false);
            }
            else if (this.dirty && !dirty) {
                this.disableSubmitReset(true);
            }
            this.dirty = dirty;
        };
        return ConfigureDialogManager;
    }());
    var DialogsManager = /** @class */ (function () {
        function DialogsManager() {
            this.createFormHallBuilders = {};
            this.editFormHallBuilders = {};
        }
        DialogsManager.prototype.getFormHallBuilder = function (formHallBuilders, dataCollectorId, then) {
            var builder = formHallBuilders[dataCollectorId];
            if (!builder) {
                formHallBuilders[dataCollectorId] = new FormHallBuilder_1.FormHallBuilder({ baseUrl: dataCollectorId, formRelUrl: '/form', uniqueId: dataCollectorId }, then);
            }
            else {
                then(builder);
            }
        };
        DialogsManager.prototype.createSourceDialog = function (dataCollectorId, onSaveSuccess) {
            this.getFormHallBuilder(this.createFormHallBuilders, dataCollectorId, function (builder) {
                var modalId = builder.prepareModalCreate();
                builder.setHandlers({ whenSuccess: function (request) {
                        onSaveSuccess(request);
                        $('[id="' + modalId + '"]').modal('hide'); ///will open configure source dialog 
                    } });
                $('[id="' + modalId + '"]').modal('show');
            });
        };
        DialogsManager.prototype.editSourceDialog = function (dataCollectorId, parameters, onSaveSuccess) {
            this.getFormHallBuilder(this.editFormHallBuilders, dataCollectorId, function (builder) {
                var modalId = builder.prepareModalEdit();
                builder.withData(parameters);
                builder.setHandlers({ whenSuccess: function () {
                        onSaveSuccess();
                    } });
                $('[id="' + modalId + '"]').modal('show');
            });
        };
        DialogsManager.prototype.deleteSourceDialog = function (sourceInfo, parameters, onDeleteSuccess) {
            $('body').append(DataCollectors_template_1.confirmSourceDelete(sourceInfo));
            var $modal = $('#confirm-source-delete-modal');
            $modal.find('#confirm-source-delete-button').on('click', function () {
                $modal.modal('hide');
                $.ajax({
                    url: 'source-parameters/' + parameters.sourceId + '?version=' + parameters.version,
                    method: 'DELETE',
                    success: function () {
                        onDeleteSuccess();
                    },
                    error: function (xhr, textStatus, errorThrown) {
                        $modal.trigger('server-error', [xhr, textStatus, errorThrown]);
                    }
                });
            });
            $modal.on('hidden.bs.modal', function () {
                $modal.remove();
            });
            $modal.modal('show');
        };
        DialogsManager.prototype.configureSourceDialog = function (sourceInfo, onSaveSuccess) {
            $('body').append(DataCollectors_template_1.configureSource(sourceInfo));
            var $modal = $('#configure-source-modal');
            var adapter = new ObservablesTreeAdapter('source-observables/' + sourceInfo.sourceId + '/', $modal);
            var manager = new TreeManager_1.TreeManager('#configure-source-modal [data-rm-role="observables-tree-placeholder"]', adapter, sourceInfo.observableRoot);
            new ConfigureDialogManager($modal, manager, onSaveSuccess);
            if (sourceInfo.selectedObservableDatas) {
                manager.populateWith(sourceInfo.selectedObservableDatas);
            }
            else {
                manager.updateRootCheckbox();
            }
            $modal.on('shown.bs.modal', function () {
                manager.scrollTopLeft();
            });
            $modal.on('hidden.bs.modal', function () {
                $modal.remove();
            });
            $modal.modal('show');
        };
        DialogsManager.prototype.displayAboutAgentDialog = function () {
            var full_url = $(location).attr('href');
            var index = full_url ? full_url.indexOf('/rm/') : -1;
            var service_url = '';
            if (index < 0) {
                service_url = '[http|https]://<service-host>:<service-port>';
            }
            else {
                service_url = full_url.substring(0, index);
            }
            var agent_url = service_url + '/rm/agent-jar';
            var params = {
                runCmd: 'java -jar <agent-jar-file> --serviceUrl=' + service_url + '/rm',
                curlCmd: 'curl -O -J ' + agent_url,
                wgetCmd: 'wget --content-disposition ' + agent_url
            };
            $('body').append(Nav_template_1.aboutAgentDialog(params));
            var $modal = $('#agent-help-modal');
            $modal.on('hidden.bs.modal', function () {
                $modal.remove();
            });
            $modal.modal('show');
        };
        return DialogsManager;
    }());
    exports.DialogsManager = DialogsManager;
});
