/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
define(["require", "exports", "jquery", "FormHallManager", "FormHall_template", "jquery.form", "jquery.serializejson", "jquery.validate"], function (require, exports, $, FormHallManager_1, FormHall_template_1) {
    "use strict";
    exports.__esModule = true;
    var FormHallBuilder = /** @class */ (function () {
        function FormHallBuilder(options, then) {
            var _this = this;
            this.options = options;
            $.ajax({
                context: this,
                url: options.baseUrl + options.formRelUrl,
                dataType: 'json',
                headers: {
                    'Cache-Control': 'max-age=0'
                },
                success: function (formModel) {
                    _this.formModel = formModel;
                    _this.formModel.baseUrl = options.baseUrl;
                    _this.formModel.uniqueId = options.uniqueId;
                    _this.formModel.whenSuccess = options.handlers ? options.handlers.whenSuccess : undefined;
                    _this.formModel.whenError = options.handlers ? options.handlers.whenError : undefined;
                    _this.formModel.customSubmit = options.handlers ? options.handlers.customSubmit : undefined;
                    then(_this);
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    console.error(textStatus);
                }
            });
        }
        FormHallBuilder.prototype.setHandlers = function (handlers) {
            this.manager.setHandlers(handlers);
        };
        FormHallBuilder.prototype.setupModalListeners = function (modalSelector) {
            var _this = this;
            $(modalSelector).on('show.bs.modal', function () {
                _this.manager.onShow();
            }).on('shown.bs.modal', function () {
                _this.manager.onShown();
            }).on('hide.bs.modal', function () {
                _this.manager.onHide();
            }).on('hidden.bs.modal', function () {
                _this.manager.onHidden();
            });
        };
        FormHallBuilder.prototype.getId = function (forEdit, withModal) {
            var subid = forEdit ? 'Edit' : 'Create';
            var id = this.options.uniqueId + subid;
            return id + (withModal ? 'FormModal' : 'Form');
        };
        FormHallBuilder.prototype.createManager = function (formId, modalId) {
            this.manager = new FormHallManager_1.FormHallManager('[id="' + formId + '"]', this.options.handlers);
            if (modalId) {
                this.setupModalListeners('[id="' + modalId + '"]');
            }
            this.manager.installValidate();
            this.manager.onUpdateData();
            if (!modalId) {
                this.manager.onShow();
            }
        };
        FormHallBuilder.prototype.prepareModal = function (forEdit) {
            var modalHtml = forEdit ? FormHall_template_1.modalEdit(this.formModel) : FormHall_template_1.modalCreate(this.formModel);
            var formId = this.getId(forEdit, false);
            var modalId = this.getId(forEdit, true);
            var $existing = $('[id="' + modalId + '"]');
            if ($existing.length) {
                $existing.replaceWith(modalHtml);
            }
            else {
                $('body').append(modalHtml);
            }
            this.createManager(formId, modalId);
            return modalId;
        };
        /** Returns the id attribute of the bootstrap modal that can then be shown with $('#' + id).modal('show'); */
        FormHallBuilder.prototype.prepareModalCreate = function () {
            return this.prepareModal(false);
        };
        /** Returns the id attribute of the bootstrap modal that can then be shown with $('#' + id).modal('show'); */
        FormHallBuilder.prototype.prepareModalEdit = function () {
            return this.prepareModal(true);
        };
        FormHallBuilder.prototype.showForm = function (placeholderSel, forEdit) {
            var $placeholder = $(placeholderSel);
            if ($placeholder.length > 0) {
                var formHtml = forEdit ? FormHall_template_1.formEdit(this.formModel) : FormHall_template_1.formCreate(this.formModel);
                var formId = this.getId(forEdit, false);
                var $existing = $placeholder.find('[id="' + formId + '"]');
                if ($existing.length) {
                    $existing.replaceWith(formHtml);
                }
                else {
                    $(placeholderSel).append(formHtml);
                }
                this.createManager(formId);
                return formId;
            }
            else {
                console.error('placeholder with selector ' + placeholderSel + ' not found; cannot append form');
            }
        };
        /** Returns the id attribute of the form element that is appended into the placeholder element */
        FormHallBuilder.prototype.showFormCreate = function (placeholderSel) {
            return this.showForm(placeholderSel, false);
        };
        /** Returns the id attribute of the form element that is appended into the placeholder element */
        FormHallBuilder.prototype.showFormEdit = function (placeholderSel) {
            return this.showForm(placeholderSel, true);
        };
        /** To be called after prepareModal...() or showForm...() */
        FormHallBuilder.prototype.withData = function (data) {
            this.manager.populateWith(data);
        };
        return FormHallBuilder;
    }());
    exports.FormHallBuilder = FormHallBuilder;
});
