/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
define(["require", "exports", "jquery", "Nav_template", "CascadableLayoutManager", "DataCollectorsLayout", "AgentsLayout", "Messages", "Constants", "x-editable"], function (require, exports, $, Nav_template_1, CascadableLayoutManager_1, DataCollectorsLayout_1, AgentsLayout_1, Messages_1, Constants_1) {
    "use strict";
    exports.__esModule = true;
    var Main = /** @class */ (function () {
        function Main() {
            var _this = this;
            this.layoutManagerRoot = new CascadableLayoutManager_1.CascadableLayoutManager();
            this.newestKnownEventTimestamp = 0;
            if (window.document['documentMode']) { // IE up to 11
                $.ajaxSetup({ cache: false });
            }
            new Constants_1.Constants();
            //translation messages must be loaded first and before evaluating templates (layouts)
            this.messages = new Messages_1.Messages(function () {
                window['messages'] = _this.messages;
                // Setup layout
                _this.dataCollectorsLayout = new DataCollectorsLayout_1.DataCollectorsLayout();
                // Setup nav placeholders
                $('body').append(Nav_template_1.nav({ 'data-collectors-layout': _this.dataCollectorsLayout, 'helpUrl': Constants_1.Constants.getHelpUrl() }));
                _this.layoutManagerRoot.addLayoutParticipant(_this.dataCollectorsLayout);
                _this.layoutManagerRoot.addLayoutParticipant(new AgentsLayout_1.AgentsLayout());
                // Start layout
                _this.loadTenantName();
                _this.layoutManagerRoot.refresh();
                // Setup global event handlers
                _this.setupEventListeners();
                // Start listening to refresh events 
                _this.gatherServerEvents();
            });
            //x-editable plugin switched to inline mode (or popup)
            $.fn.editable.defaults.mode = 'inline'; //'popup'; //'inline';
        }
        Main.prototype.loadTenantName = function () {
            $.ajax({
                url: 'tenant-name',
                dataType: 'text',
                success: function (tenantName) {
                    var $tenantElement = $('[data-rm-role="tenant-name"]');
                    if (tenantName !== '') {
                        $tenantElement.html(tenantName);
                        $tenantElement.closest('[data-rm-role="menu-tenant"]').removeClass('hidden');
                    }
                },
                error: function (xhr, textStatus, errorThrown) {
                    $('[data-rm-role="tenant-name"]').trigger('server-error', [xhr, textStatus, errorThrown]);
                }
            });
        };
        Main.prototype.setupEventListeners = function () {
            var _this = this;
            var $body = $('body');
            $body.on('click', function (event) {
                return _this.onEvent(event);
            }).on('keyup', function (event) {
                var $target = $(event.target);
                if ($target.is('input[type="checkbox"],input[type="radio"]') && event.key === ' ') {
                    // Space bar on a checkbox or radio
                    return _this.onEvent(event);
                }
                else if (event.which == 13 || event.keyCode == 13) {
                    // Enter key
                    return _this.onEvent(event);
                }
            }).on('hide.bs.collapse', function (event) {
                return _this.onEvent(event);
            }).on('hidden.bs.collapse', function (event) {
                return _this.onEvent(event);
            }).on('show.bs.collapse', function (event) {
                return _this.onEvent(event);
            }).on('shown.bs.collapse', function (event) {
                return _this.onEvent(event);
            }).on('source-container-updated', function (event) {
                return _this.onEvent(event);
            }).on('data-collector-updated', function (event) {
                return _this.onEvent(event);
            }).on('server-error', function (event, xhr, textStatus, errorThrown) {
                event.data = event.data || {};
                event.data.xhr = xhr;
                event.data.textStatus = textStatus;
                event.data.errorThrown = errorThrown;
                return _this.onEvent(event);
            });
        };
        Main.prototype.onEvent = function (event) {
            var data = event.data || {};
            var $target = $(event.target);
            while ($target.length > 0 && !$target.attr('data-rm-role')) {
                $target = $target.parent();
            }
            data.$target = $target;
            data.role = $target.data('rmRole');
            event.data = data;
            if (event.type === 'server-error') {
                console.error('got the event with status ' + event.data.xhr.status);
                // Temp: show an alert
                alert('Service has returned status code ' + event.data.xhr.status);
            }
            this.layoutManagerRoot.decorateEventData(event);
            return this.layoutManagerRoot.onEvent(event);
        };
        Main.prototype.gatherServerEvents = function () {
            var _this = this;
            $.ajax({
                url: 'events-channel' + (this.newestKnownEventTimestamp === 0 ? '' : '?newestKnownEvent=' + this.newestKnownEventTimestamp),
                dataType: 'json',
                timeout: 60000,
                error: function (jqXHR, textStatus) {
                    // In case of error, rearm in a few seconds
                    setTimeout(function () {
                        _this.gatherServerEvents();
                    }, 10000);
                },
                success: function (events, textStatus, jqXHR) {
                    var serviceClockString = jqXHR.getResponseHeader('X-RMS-Service-Clock');
                    if (serviceClockString) {
                        var serviceClock = Number(serviceClockString);
                        window['serviceClockSkew'] = new Date().getTime() - serviceClock;
                        //console.log('serviceClockSkew: ' + window['serviceClockSkew']);
                        if (_this.newestKnownEventTimestamp === 0) {
                            _this.newestKnownEventTimestamp = serviceClock;
                        }
                    }
                    else {
                        window['serviceClockSkew'] = 0;
                    }
                    if (_this.newestKnownEventTimestamp === 0) {
                        _this.newestKnownEventTimestamp = new Date().getTime();
                    }
                    if (events && events.length && events.length > 0) {
                        // TODO: call participants refresh() depending on the events or publish them as js events?
                        //console.log(JSON.stringify(events));
                        for (var _i = 0, events_1 = events; _i < events_1.length; _i++) {
                            var event_1 = events_1[_i];
                            // Event operations are defined in enum com.hcl.test.rm.service.core.events.RMEventInfo.EventOperation
                            // Some have parameters, some others not, look for usages of the operation constants to get a list of provided parameters
                            _this.layoutManagerRoot.onServerEvent(event_1);
                        }
                        // Update timestamp with new newest
                        _this.newestKnownEventTimestamp = events[events.length - 1].timestamp;
                    }
                    //console.log('newestKnownEvent: ' + this.newestKnownEvent);
                    // Rearm
                    setTimeout(function () {
                        _this.gatherServerEvents();
                    });
                }
            });
        };
        return Main;
    }());
    new Main();
});
