/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
define(["require", "exports", "jquery", "d3", "moment-with-locales"], function (require, exports, $, d3) {
    "use strict";
    exports.__esModule = true;
    var Messages = /** @class */ (function () {
        function Messages(callback) {
            this.messages = null;
            //copied from Unified Reporting translation.service.ts:
            this.localeID = 'undefined'; //locale id returned by server
            this.loadMessages(callback);
            Messages.instance = this;
        }
        Messages.prototype.loadMessages = function (callback) {
            var _this = this;
            $.getJSON('ui/tr')
                .done(function (data) {
                if (data) {
                    _this.messages = data.webui; //translated messages for interface
                    _this.localeID = data.LocaleID;
                    _this.updatePluralLocale();
                    if (data.jqueryValidatorMessages) {
                        _this.updateJQueryValidationMessages(data.jqueryValidatorMessages);
                    }
                    if (data.jqueryValidatorMethods) {
                        _this.updateJQueryValidationMethods(data.jqueryValidatorMethods);
                    }
                    // time to set moment locale
                    moment.locale(_this.localeID); //time to set locale in moment for calendar, dates, ...
                    if (data.d3Format) {
                        _this.updateD3FormatLocale(data.d3Format);
                    }
                    if (data.d3TimeFormat) {
                        _this.updateD3TimeFormatLocale(data.d3TimeFormat);
                    }
                }
                callback();
            })
                .fail(function (jqxhr, textStatus, error) {
                console.error("ERROR: jqxhr=", jqxhr, " textStatus=", textStatus, " error=", error);
            });
        };
        /** @return the locale id solved by server, may be undefined */
        Messages.prototype.getLocaleID = function () {
            return this.localeID;
        };
        /** @return translation for key, optionally with parameter bound to {0}... arguments in value */
        Messages.prototype.get = function (key) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (this.messages && this.messages[key]) {
                if (args.length) {
                    return this.bind.apply(this, [this.messages[key]].concat(args));
                }
                else {
                    return this.messages[key]; //no bind
                }
            }
            else {
                return key;
            }
        };
        /** @return translation for key using .pluralX key suffix (depends on localeID), see messages.properties for details */
        Messages.prototype.getPlural = function (key, num) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            //plural require localID:
            var suffix = this.getPluralSuffixFunc(num);
            var plural_key = key + suffix;
            //hope messages are loaded here !
            if (!this.messages) {
                return key;
            }
            //prepend num in var args used in following bind()
            var args2 = [];
            args2.push(num);
            for (var i = 0; i < args.length; i++) {
                args2.push(args[i]);
            }
            if (this.messages[plural_key]) {
                var msg = this.messages[plural_key];
                if (msg && args2.length) {
                    return this.bind.apply(this, [msg].concat(args2));
                }
                else {
                    return msg;
                }
            }
            //fallback for compatibility and no-yet translated files (or optional .plural1 locale): use simple key without suffix
            else if (this.messages[key]) {
                var msg = this.messages[key];
                if (msg && args2.length) {
                    return this.bind.apply(this, [msg].concat(args2));
                }
                else {
                    return msg;
                }
            }
            return key;
        };
        /** @return raw value of message matching key, no parameter bind are processed here */
        Messages.prototype.getNoBind = function (key) {
            if (this.messages && this.messages[key]) {
                return this.messages[key];
            }
            else {
                return key;
            }
        };
        Messages.Get = function (key) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (Messages.instance) {
                return (_a = Messages.instance).get.apply(_a, [key].concat(args));
            }
            else {
                return key;
            }
            var _a;
        };
        Messages.GetPlural = function (key, num) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            if (Messages.instance) {
                return (_a = Messages.instance).getPlural.apply(_a, [key, num].concat(args));
            }
            else {
                return key;
            }
            var _a;
        };
        Messages.prototype.bind = function (value) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var ret = value;
            if (args instanceof Array) {
                for (var i = 0; i < args.length; i++) {
                    // Cast arg to <any> to have automatic conversion of null to "null" (null.toString() fails)
                    ret = ret.replace("{" + i + "}", args[i]);
                }
            }
            return ret;
        };
        Messages.prototype.updatePluralLocale = function () {
            if (!this.localeID) {
                //default locale: 'en'
                this.getPluralSuffixFunc = this.pluralRule_en;
            }
            else {
                var loc4 = this.localeID.substring(0, 4);
                var loc2 = this.localeID.substring(0, 2);
                if ("pt_BR" === loc4) {
                    this.getPluralSuffixFunc = this.pluralRule_fr;
                }
                else if ("en" === loc2 || "es" === loc2 || "it" === loc2 || "de" === loc2 || "hu" === loc2) {
                    this.getPluralSuffixFunc = this.pluralRule_en;
                }
                else if ("fr" === loc2 || "tr" === loc2) {
                    this.getPluralSuffixFunc = this.pluralRule_fr;
                }
                else if ("cs" === loc2) {
                    this.getPluralSuffixFunc = this.pluralRule_cs;
                }
                else if ("pl" === loc2) {
                    this.getPluralSuffixFunc = this.pluralRule_pl;
                }
                else if ("ru" === loc2) {
                    this.getPluralSuffixFunc = this.pluralRule_ru;
                }
                else { // ja, ko zh, zh_CN, zh_HK, zh_TW
                    this.getPluralSuffixFunc = this.pluralRule_others;
                }
            }
        };
        //see Messages.properties for explanations
        //source: //doc.qt.io/archives/qq/qq19-plurals.html
        //   and: //docs.translatehouse.org/projects/localization-guide/en/latest/l10n/pluralforms.html
        Messages.prototype.pluralRule_en = function (n) {
            if (n == 1)
                return ".plural1";
            else
                return ".plural2";
        };
        Messages.prototype.pluralRule_fr = function (n) {
            if (n < 2)
                return ".plural1";
            else
                return ".plural2";
        };
        Messages.prototype.pluralRule_cs = function (n) {
            if (n % 100 == 1)
                return ".plural1";
            else if (n % 100 >= 2 && n % 100 <= 4)
                return ".plural2";
            else
                return ".plural3";
        };
        Messages.prototype.pluralRule_pl = function (n) {
            if (n == 1)
                return ".plural1";
            else if (n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 > 20))
                return ".plural2";
            else
                return ".plural3";
        };
        Messages.prototype.pluralRule_ru = function (n) {
            if (n % 10 == 1 && n % 100 != 11)
                return ".plural1";
            else if (n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 > 20))
                return ".plural2";
            else
                return ".plural3";
        };
        Messages.prototype.pluralRule_others = function (n) {
            return ".plural1";
        };
        //end-of-copied code
        Messages.prototype.updateJQueryValidationMessages = function (url) {
            if (!url)
                return; //required
            $.getScript(url, function () {
                // N/A
            })["catch"](function () {
                console.error("Error: cannot load messages: " + url);
            });
        };
        Messages.prototype.updateJQueryValidationMethods = function (url) {
            if (!url)
                return; //required
            $.getScript(url, function () {
                // N/A
            })["catch"](function () {
                console.error("Error: cannot load methods: " + url);
            });
        };
        Messages.prototype.updateD3TimeFormatLocale = function (url) {
            //first try: complete:
            $.getJSON(url)
                .done(function (data) {
                var locale = data;
                d3.timeFormatDefaultLocale(locale);
            })
                .fail(function (jqxhr, textStatus, error) {
                console.log("Error: D3 time format: ", error);
                //no fallback, continue with current time format
            });
        };
        Messages.prototype.updateD3FormatLocale = function (url) {
            //first try: complete:
            $.getJSON(url)
                .done(function (data) {
                var locale = data;
                d3.formatDefaultLocale(locale);
            })
                .fail(function (jqxhr, textStatus, error) {
                console.log("Error: D3 time format: ", error);
                //no fallback, continue with current time format
            });
        };
        Messages.instance = null;
        return Messages;
    }());
    exports.Messages = Messages;
});
