/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "jquery", "LayoutParticipant", "ObservationsChart", "DataCollectors_template", "Messages"], function (require, exports, $, LayoutParticipant_1, ObservationsChart_1, DataCollectors_template_1, Messages_1) {
    "use strict";
    exports.__esModule = true;
    var SourceLayout = /** @class */ (function (_super) {
        __extends(SourceLayout, _super);
        function SourceLayout(dialogsManager, sourceInfo, cascadableLayoutManager) {
            var _this = _super.call(this, 'sources?sourceId=' + sourceInfo.sourceId, ['edit-source', 'delete-source', 'configure-source', 'collapsible-sources-container', 'details-container', 'extended-layout', 'responsive-layout'], ['SOURCE_MODIFIED', 'SOURCE_STATUS_CHANGED', 'SOURCE_RENAMED', 'SOURCE_REMOVED', 'SELECTED_OBSERVABLES_CHANGED', 'LAST_OBSERVED_ON_CHANGED']) || this;
            _this.dialogsManager = dialogsManager;
            _this.sourceInfo = sourceInfo;
            _this.cascadableLayoutManager = cascadableLayoutManager;
            _this.detailsShown = false;
            cascadableLayoutManager.addLayoutParticipant(_this);
            return _this;
        }
        SourceLayout.prototype.refresh = function (onComplete) {
            var _this = this;
            if (onComplete === void 0) { onComplete = null; }
            $.ajax({
                url: this.url,
                dataType: 'json',
                headers: {
                    'Cache-Control': 'max-age=0'
                },
                success: function (sourceInfos) {
                    _this.display(sourceInfos[0], onComplete);
                },
                error: function (xhr, textStatus, errorThrown) {
                    $('[data-rm-role="source-container"][data-rm-source-id="' + _this.sourceInfo.sourceId + '"]').trigger('server-error', [xhr, textStatus, errorThrown]);
                }
            });
        };
        SourceLayout.prototype.display = function (sourceInfo, onComplete) {
            if (onComplete === void 0) { onComplete = null; }
            this.sourceInfo = sourceInfo;
            var $existing = $('[data-rm-role="source-container"][data-rm-source-id="' + this.sourceInfo.sourceId + '"]');
            if ($existing.length > 0) {
                if (this.detailsShown) {
                    this.toggleObservationsChart(null, false);
                }
                $existing.replaceWith(DataCollectors_template_1.source(this.sourceInfo));
                if (this.detailsShown) {
                    var $sourceLayout = $('[data-rm-role="source-container"][data-rm-source-id="' + this.sourceInfo.sourceId + '"]');
                    var $expandCollapseDetails = $sourceLayout.find('[data-rm-role="expand-collapse-details"]');
                    if (!$expandCollapseDetails.hasClass('hidden')) {
                        var $toggle = $expandCollapseDetails.find('[data-rm-toggle-classes][data-rm-toggle-classes!=""]');
                        this.updateToggleState($toggle);
                        var $lastObservedOn = $sourceLayout.find('[data-rm-role="last-observed-on"]');
                        $lastObservedOn.addClass('hidden');
                        $($expandCollapseDetails.data('target')).toggleClass('in');
                        this.toggleObservationsChart($sourceLayout, true);
                    }
                    else {
                        this.detailsShown = false;
                    }
                }
            }
            else {
                this.getParent().append(DataCollectors_template_1.source(this.sourceInfo));
            }
            this.makeTitleEditable();
            this.triggerUpdatedEvent();
            if (onComplete) {
                setTimeout(function () {
                    onComplete();
                });
            }
        };
        SourceLayout.prototype.updateLayout = function () {
            if (this.observationsChart && this.detailsShown) {
                this.observationsChart.updateWidth();
            }
        };
        SourceLayout.prototype.getParent = function () {
            return $('[data-rm-data-collector-id="' + this.sourceInfo.dataCollectorId + '"] [data-rm-role="sources-container"]');
        };
        SourceLayout.prototype.triggerDataCollectorUpdatedEvent = function () {
            this.getParent().trigger('data-collector-updated');
        };
        SourceLayout.prototype.triggerUpdatedEvent = function () {
            this.getParent().trigger('source-container-updated');
        };
        SourceLayout.prototype.loadParameters = function (then) {
            var _this = this;
            $.ajax({
                url: 'source-parameters/' + this.sourceInfo.sourceId,
                dataType: 'json',
                headers: {
                    'Cache-Control': 'max-age=0'
                },
                success: function (parameters) {
                    then(parameters);
                },
                error: function (xhr, textStatus, errorThrown) {
                    $('[data-rm-role="source-container"][data-rm-source-id="' + _this.sourceInfo.sourceId + '"]').trigger('server-error', [xhr, textStatus, errorThrown]);
                }
            });
        };
        SourceLayout.prototype.onEvent = function (event) {
            switch (event.type) {
                case 'show':
                case 'shown':
                case 'hide':
                case 'hidden':
                    switch (event.data.role) {
                        case 'details-container':
                            if (event.data.sourceId === this.sourceInfo.sourceId) {
                                return this.toggleDetails(event, true);
                            }
                            break;
                        case 'collapsible-sources-container':
                            if (event.data.dataCollectorId === this.sourceInfo.dataCollectorId) {
                                return this.toggleDetails(event, false);
                            }
                            break;
                    }
                    break;
                default:
                    if (event.data.sourceId === this.sourceInfo.sourceId) {
                        switch (event.data.role) {
                            case 'edit-source':
                                this.onEditSource(event.data.dataCollectorId);
                                break;
                            case 'delete-source':
                                this.onDeleteSource();
                                break;
                            case 'configure-source':
                                this.onConfigureSource();
                                break;
                        }
                        return false;
                    }
                    switch (event.data.role) {
                        case 'extended-layout':
                        case 'responsive-layout':
                            this.updateLayout();
                            break;
                    }
                    break;
            }
            return true;
        };
        SourceLayout.prototype.onEditSource = function (dataCollectorId) {
            var _this = this;
            this.loadParameters(function (parameters) {
                _this.dialogsManager.editSourceDialog(dataCollectorId, parameters, function () {
                    _this.refresh();
                });
            });
        };
        SourceLayout.prototype.onDeleteSource = function () {
            var _this = this;
            this.loadParameters(function (parameters) {
                _this.dialogsManager.deleteSourceDialog(_this.sourceInfo, parameters, function () {
                    _this["delete"]();
                });
            });
        };
        SourceLayout.prototype.onConfigureSource = function () {
            var _this = this;
            this.dialogsManager.configureSourceDialog(this.sourceInfo, function () {
                _this.refresh();
            });
        };
        SourceLayout.prototype.toggleDetails = function (event, animate) {
            if (animate) {
                this.animateToggleDetails(event.data.$sourceLayout, event.type);
            }
            var $sourceLayout = event.data.$sourceLayout;
            if (!$sourceLayout) {
                $sourceLayout = event.data.$dataCollectorLayout.find('[data-rm-source-id="' + this.sourceInfo.sourceId + '"]');
            }
            switch (event.type) {
                case 'shown':
                case 'hide':
                    if (animate) {
                        this.detailsShown = event.type === 'shown';
                        this.toggleObservationsChart($sourceLayout, this.detailsShown);
                    }
                    else if (this.detailsShown) {
                        this.toggleObservationsChart($sourceLayout, event.type === 'shown');
                    }
                    break;
                case 'show':
                case 'hidden':
                    var $lastObservedOn = $sourceLayout.find('[data-rm-role="last-observed-on"]');
                    if (event.type === 'hidden') {
                        $lastObservedOn.attr('data-rm-time-value', this.sourceInfo.lastObservedOn);
                        if (this.sourceInfo.lastObservedOn) {
                            $lastObservedOn.removeClass('hidden');
                        }
                        this.triggerUpdatedEvent();
                    }
                    else {
                        $lastObservedOn.addClass('hidden');
                    }
                    break;
            }
            return !animate;
        };
        SourceLayout.prototype.animateToggleDetails = function ($target, eventType) {
            var $toggle = $target.find('[data-rm-toggle-classes][data-rm-toggle-classes!=""]');
            switch (eventType) {
                case 'show':
                    $toggle.addClass('icon-animate-qspin');
                    break;
                case 'shown':
                    $toggle.removeClass('icon-animate-qspin');
                    break;
                case 'hide':
                    $toggle.addClass('icon-animate-rqspin');
                    break;
                case 'hidden':
                    $toggle.removeClass('icon-animate-rqspin');
                    break;
            }
            switch (eventType) {
                case 'show':
                case 'hide':
                    this.updateToggleState($toggle);
                    break;
            }
        };
        SourceLayout.prototype.updateToggleState = function ($toggle) {
            var classes = $toggle.data('rmToggleClasses').split(' ');
            $toggle.toggleClass(classes[0]);
            $toggle.toggleClass(classes[1]);
        };
        SourceLayout.prototype.toggleObservationsChart = function ($sourceLayout, show) {
            if (show) {
                if (!this.observationsChart || !this.observationsChart.startDisplay()) {
                    var $detailsContainer = $sourceLayout.find('[data-rm-role="details-container"]');
                    this.observationsChart = new ObservationsChart_1.ObservationsChart(this.sourceInfo, 'source-observations/' + this.sourceInfo.sourceId, $detailsContainer);
                }
                this.observationsChart.startDisplay();
            }
            else {
                if (this.observationsChart) {
                    this.observationsChart.stopDisplay();
                }
            }
        };
        SourceLayout.prototype["delete"] = function () {
            if (this.observationsChart) {
                this.observationsChart.stopDisplay();
                this.observationsChart = null;
            }
            var $existing = $('[data-rm-role="source-container"][data-rm-source-id="' + this.sourceInfo.sourceId + '"]');
            $existing.remove();
            this.cascadableLayoutManager.removeLayoutParticipant(this);
            this.triggerDataCollectorUpdatedEvent();
        };
        SourceLayout.prototype.makeTitleEditable = function () {
            var _this_ = this;
            var $source = $('[data-rm-role="source-container"][data-rm-source-id="' + this.sourceInfo.sourceId + '"]');
            var $title = $source.find('#source-title');
            $title.editable({
                pk: 1,
                //cannot use url directly as editable send json versus RMService expect the name as body text/plain
                url: function (params) {
                    var urlRename = 'source-title/' + _this_.sourceInfo.sourceId + '?version=' + _this_.sourceInfo.parametersVersion;
                    var d = $.Deferred();
                    if ($.trim(params.value) == '') {
                        return d.reject(Messages_1.Messages.Get('SL_EMPTY_NAME_ERROR'));
                    }
                    else {
                        $.ajax({
                            url: urlRename,
                            type: 'POST',
                            dataType: 'text',
                            contentType: 'text/plain; charset=UTF-8',
                            data: params.value,
                            success: function (code_html, status, jqXHR) {
                                // success renaming ? can incr. version locally, avoiding full refresh
                                if (jqXHR.status === 200) {
                                    _this_.sourceInfo.parametersVersion++;
                                    _this_.sourceInfo.title = params.value;
                                } // 204 means the title was the same
                                d.resolve();
                            },
                            error: function (xhr, textStatus, errorThrown) {
                                var msg = 'Error';
                                if (xhr && xhr.responseText) {
                                    msg = xhr.responseText; //message is enough
                                }
                                else if (xhr && xhr.status) {
                                    msg += ' - ' + xhr.status; //otherwise display status code if any
                                }
                                d.reject(msg);
                                // TODO: source modified concurrently: how does the end-user refreshes?
                            }
                        });
                        return d.promise();
                    }
                }
            });
        };
        SourceLayout.prototype.onServerEvent = function (event) {
            var _this = this;
            if (event.parameters && event.parameters.sourceId === this.sourceInfo.sourceId) {
                var $sourceLayout = $('[data-rm-role="source-container"][data-rm-source-id="' + this.sourceInfo.sourceId + '"]');
                switch (event.operation) {
                    case 'SOURCE_MODIFIED':
                    case 'SOURCE_STATUS_CHANGED':
                    case 'SELECTED_OBSERVABLES_CHANGED':
                        this.refresh(function () {
                            var $sourceLayout = $('[data-rm-role="source-container"][data-rm-source-id="' + _this.sourceInfo.sourceId + '"]');
                            _this.animateRefreshedElement($sourceLayout);
                        });
                        break;
                    case 'SOURCE_REMOVED':
                        this["delete"]();
                        break;
                    case 'SOURCE_RENAMED':
                        var $title = $sourceLayout.find('#source-title');
                        var isEditing = $title.hasClass('editable-open');
                        if (isEditing) {
                            console.log('editable is open but modified concurrently');
                            // TODO: currently, the editable closes and the new title is shown: is it acceptable?
                        }
                        $title.editable('destroy');
                        $title.text(event.parameters.newTitle);
                        this.makeTitleEditable();
                        this.sourceInfo.parametersVersion = event.parameters.newVersion;
                        this.animateRefreshedElement($title);
                        break;
                    case 'LAST_OBSERVED_ON_CHANGED':
                        this.sourceInfo.lastObservedOn = event.parameters.newLastObservedOn;
                        var $lastObservedOn = $sourceLayout.find('[data-rm-role="last-observed-on"]');
                        $lastObservedOn.attr('data-rm-time-value', this.sourceInfo.lastObservedOn);
                        this.triggerUpdatedEvent();
                        this.animateRefreshedElement($lastObservedOn);
                        break;
                }
                return false;
            }
            return true;
        };
        SourceLayout.prototype.animateRefreshedElement = function ($element) {
            if ($element[0]['refresh-animation-timer']) {
                clearTimeout($element[0]['refresh-animation-timer']);
            }
            $element.addClass('highlight-update');
            $element.addClass('in');
            $element[0]['refresh-animation-timer'] = setTimeout(function () {
                $element.removeClass('in');
                $element.addClass('out');
                $element[0]['refresh-animation-timer'] = setTimeout(function () {
                    $element.removeClass('highlight-update');
                    $element.removeClass('out');
                }, 2000);
            }, 5000);
        };
        return SourceLayout;
    }(LayoutParticipant_1.LayoutParticipant));
    exports.SourceLayout = SourceLayout;
});
