/****************************************************************
 Licensed Materials - Property of IBM and/or HCL
(c) Copyright HCL Technologies Ltd. 2018.  All Rights Reserved.

 Note to U.S. Government Users Restricted Rights:
 Use, duplication or disclosure restricted by GSA ADP Schedule:
 Contract with IBM Corp.
*****************************************************************/
define(["require", "exports", "jquery", "Tree_template"], function (require, exports, $, Tree_template_1) {
    "use strict";
    exports.__esModule = true;
    function cancelEvent(event) {
        event.preventDefault ? event.preventDefault() : event.returnValue = false;
        if (event.stopPropagation) {
            event.stopPropagation();
        }
        return false;
    }
    var TreeAdapter = /** @class */ (function () {
        function TreeAdapter() {
        }
        // Refresh/Lazyness:
        /** loads a lazy or refreshable item, must call then(item) in case of success */
        TreeAdapter.prototype.loadItem = function (id, complete) {
            // Default implementation does nothing
            complete({ 'tree-stub-item': true });
        };
        return TreeAdapter;
    }());
    exports.TreeAdapter = TreeAdapter;
    var TreeManager = /** @class */ (function () {
        function TreeManager(placeholderSelector, adapter, rootItem) {
            this.placeholderSelector = placeholderSelector;
            this.adapter = adapter;
            this.rootItem = rootItem;
            rootItem['tree-adapter'] = adapter;
            $(placeholderSelector).html(Tree_template_1.tree(rootItem));
            this.$treeContainer = $(placeholderSelector).find('[data-tree-role="container"]');
            this.setupEventListeners();
        }
        TreeManager.prototype.getChildrenRole = function ($treeItem) {
            return $treeItem.find('[data-tree-role="children"]').first();
        };
        TreeManager.prototype.getItemFromInner = function ($element) {
            return $element.closest('[data-tree-role="item"]');
        };
        TreeManager.prototype.getParentItemFromInner = function ($element) {
            return $element.parents('[data-tree-role="item"]').eq(1);
        };
        TreeManager.prototype.toggleNode = function ($target) {
            var $childrenRole = $target.siblings('[data-tree-role="children"]');
            var newState = $childrenRole.hasClass('hidden');
            $childrenRole.toggleClass('hidden');
            $target.toggleClass('collapse-icon', newState).toggleClass('expand-icon', !newState);
        };
        TreeManager.prototype.setupEventListeners = function () {
            var _this = this;
            this.$treeContainer.on('click', function (event) {
                return _this.onEvent(event);
            });
            this.$treeContainer.on('keyup', function (event) {
                var $target = $(event.target);
                if ($target.is('input[type="checkbox"]') && event.key === ' ') {
                    return _this.onEvent(event);
                }
                else if (event.which == 13 || event.keyCode == 13) {
                    return _this.onEvent(event);
                }
                // TODO: arrow keys to navigate the tree
            });
            this.$treeContainer.on('mouseover', function (event) {
                var $target = $(event.target);
                switch ($target.attr('data-tree-role')) {
                    case 'load-explanation-on-hover':
                        if (!$target[0]['hoverTimer']) {
                            $target[0]['hoverTimer'] = setTimeout(function () {
                                $target[0]['hoverTimer'] = null;
                                var $treeItem = _this.getItemFromInner($target);
                                _this.loadOrRefreshItem($treeItem, null, true);
                            }, 250);
                            $target.off('mouseleave').on('mouseleave', function () {
                                if ($target[0]['hoverTimer']) {
                                    clearTimeout($target[0]['timer']);
                                    $target[0]['timer'] = null;
                                }
                            });
                        }
                        break;
                }
            });
            this.installExplanationOnHover();
        };
        TreeManager.prototype.onEvent = function (event) {
            var _this = this;
            var $target = $(event.target);
            while ($target.length > 0 && !$target.attr('data-tree-role')) {
                $target = $target.parent();
            }
            switch ($target.attr('data-tree-role')) {
                case 'collapse-expand':
                    var $treeItem = this.getItemFromInner($target);
                    var childrenRole = this.getChildrenRole($treeItem);
                    if (childrenRole.length > 0) {
                        this.toggleNode($target);
                    }
                    else {
                        this.loadOrRefreshItem($treeItem, function ($treeItem) {
                            setTimeout(function () {
                                _this.updateContainerCheckbox($treeItem.find('[data-tree-role="container-checkbox"]').first());
                            });
                        });
                    }
                    return cancelEvent(event);
                case 'container-checkbox':
                    this.containerCheckboxClicked($target);
                    break;
                case 'leaf-checkbox':
                    this.updateParents($target);
                    break;
                case 'refresh':
                case 'refresh-icon':
                    var $treeItem = this.getItemFromInner($target);
                    this.loadOrRefreshItem($treeItem, null, true);
                    break;
            }
        };
        TreeManager.prototype.installExplanationOnHover = function () {
            this.$treeContainer.find('[data-tree-role="explanation-on-hover"]').each(function (index, element) {
                $(element).tooltip();
                $(element).attr('data-tree-role', 'explanation-on-hover-installed');
            });
        };
        TreeManager.prototype.updateContainerCheckbox = function ($containerCheckbox) {
            var $childrenCheckboxes = this.getItemFromInner($containerCheckbox).find('[data-tree-role="leaf-checkbox"]');
            if ($childrenCheckboxes.length > 0) {
                var and = true, or = false;
                $childrenCheckboxes.each(function (index, element) {
                    var checked = $(element).prop('checked');
                    and = and && checked;
                    or = or || checked;
                });
                if (and) {
                    $containerCheckbox.prop('checked', true);
                    $containerCheckbox.prop('indeterminate', false);
                }
                else if (!or) {
                    $containerCheckbox.prop('checked', false);
                    $containerCheckbox.prop('indeterminate', false);
                }
                else {
                    $containerCheckbox.prop('indeterminate', true);
                }
            }
            this.updateParents($containerCheckbox);
        };
        TreeManager.prototype.updateParents = function ($checkbox) {
            var $parentCheckbox = this.getParentItemFromInner($checkbox).find('[data-tree-role="item-label"]')
                .first().find('[data-tree-role="container-checkbox"]');
            if ($parentCheckbox.length > 0) {
                this.updateContainerCheckbox($parentCheckbox);
            }
            if ($checkbox.data('treeRole') === 'container-checkbox') {
                this.disableContainerCheckboxIfNeeded($checkbox);
            }
        };
        TreeManager.prototype.containerCheckboxClicked = function ($containerCheckbox) {
            var $containerItem = this.getItemFromInner($containerCheckbox);
            var $childrenLeafCheckboxes = $containerItem.find('[data-tree-role="leaf-checkbox"]');
            $childrenLeafCheckboxes.prop('checked', $containerCheckbox.prop('checked'));
            var $childrenContainerCheckboxes = this.getChildrenRole($containerItem).find('[data-tree-role="container-checkbox"]');
            $childrenContainerCheckboxes.prop('checked', $containerCheckbox.prop('checked'));
            $childrenContainerCheckboxes.prop('indeterminate', false);
            this.updateParents($containerCheckbox);
        };
        TreeManager.prototype.loadOrRefreshItem = function ($treeItem, then, refresh) {
            var _this = this;
            if (!$treeItem.prop('loading-tree-item')) {
                $treeItem.prop('loading-tree-item', true);
                var idAttribute = $treeItem.attr('id');
                var $refreshIndicator = $treeItem.find('[data-tree-role="refresh-icon"]').first();
                if (!refresh) {
                    $refreshIndicator.removeClass('hidden');
                }
                $refreshIndicator.addClass('icon-animate-spin');
                var id = $treeItem.data('treeItemId');
                var savedState = refresh ? this.getCurrentState($treeItem) : null;
                this.adapter.loadItem(id, function (data) {
                    if (data['tree-stub-item'] === true) {
                        console.error('Missing implementation for TreeAdapter.load(id: string, complete(data: any): void)...');
                    }
                    else if (data['server-error'] === true) {
                        console.error('Server error');
                    }
                    else {
                        data['tree-adapter'] = _this.adapter;
                        $treeItem.replaceWith(Tree_template_1.treeItem(data));
                        $treeItem = $('[id="' + idAttribute + '"]');
                        if (savedState) {
                            _this.restoreState($treeItem, savedState);
                        }
                        _this.installExplanationOnHover();
                    }
                    $refreshIndicator.removeClass('icon-animate-spin');
                    if (!refresh) {
                        $refreshIndicator.addClass('hidden');
                    }
                    $treeItem.prop('loading-tree-item', false);
                    if (then) {
                        then($treeItem);
                    }
                });
            }
        };
        TreeManager.prototype.getCurrentState = function ($treeItem) {
            var _this = this;
            var state = { id: $treeItem.data('treeItemId') };
            var isContainer = $treeItem.find('[data-tree-role="collapse-expand"]').length > 0;
            if (isContainer) {
                var $children = this.getChildrenRole($treeItem);
                if ($children.length === 1) {
                    state.expanded = !$children.hasClass('hidden');
                    var $currentChildren = $children;
                    var $childrenItems = $currentChildren.children('[data-tree-role="item"]');
                    var remainingIterations = 5; // Avoid infinite loop
                    while ($childrenItems.length === 0 && remainingIterations--) {
                        $currentChildren = $currentChildren.children();
                        $childrenItems = $currentChildren.children('[data-tree-role="item"]');
                    }
                    state.children = [];
                    $childrenItems.each(function (index, element) {
                        var $element = $(element);
                        state.children.push(_this.getCurrentState($element));
                    });
                }
            }
            else {
                var $checkbox = $treeItem.find('[data-tree-role="leaf-checkbox"]').first();
                state.checkboxDefaultValue = $checkbox.attr('checked');
                state.checkboxValue = $checkbox.prop('checked');
            }
            return state;
        };
        TreeManager.prototype.restoreState = function ($treeItem, state) {
            var _this = this;
            var isContainer = $treeItem.find('[data-tree-role="collapse-expand"]').length > 0;
            if (isContainer) {
                var $target = $treeItem.find('[data-tree-role="collapse-expand"]').first();
                var childrenRole = this.getChildrenRole($treeItem);
                if (childrenRole.length > 0) {
                    this.restoreChildrenState($treeItem, state);
                }
                else if (state.expanded) {
                    this.loadOrRefreshItem($treeItem, function ($treeItem) {
                        _this.restoreChildrenState($treeItem, state);
                    });
                }
            }
            else {
                if (state.checkboxDefaultValue || state.checkboxValue) {
                    var $checkbox = $treeItem.find('[data-tree-role="leaf-checkbox"]').first();
                    if (state.checkboxDefaultValue) {
                        $checkbox.attr('checked', 'checked');
                        if (!state.checkboxValue) {
                            $checkbox.prop('checked', false);
                        }
                    }
                    if (state.checkboxValue) {
                        $checkbox.prop('checked', true);
                    }
                }
            }
        };
        TreeManager.prototype.restoreChildrenState = function ($treeItem, state) {
            if (state.children) {
                for (var index = 0; index < state.children.length; index++) {
                    var childState = state.children[index];
                    var $childItem = $treeItem.find('[data-tree-item-id="' + childState.id + '"]');
                    if ($childItem.length > 0) {
                        this.restoreState($childItem, childState);
                    }
                }
            }
            this.updateContainerCheckbox($treeItem.find('[data-tree-role="container-checkbox"]').first());
        };
        /**
         * Recompute parent checkboxes after a form reset that put leaf ones to their default values, i.e. the ones at populateWith() time.
         * Does nothing if no checkboxes.
         * Must be called after form reset has finished its normal job (i.e. in a setTimeout(() => { manager.onReset(); } of the form reset handler);
         */
        TreeManager.prototype.onReset = function () {
            var _this = this;
            var $allContainerCheckboxes = this.$treeContainer.find('[data-tree-role="container-checkbox"]');
            $allContainerCheckboxes.each(function (index, element) {
                _this.updateContainerCheckbox($(element)); // TODO: Not optimized
            });
        };
        /**
         * Given an array of ids (corresponding to the id of a leaf checkbox), check them all on and leave the other ones alone.
         * Example: treeManager.populateWith(['id1', 'id2'])
         */
        TreeManager.prototype.populateWith = function (data) {
            var _this = this;
            var parentIds = [];
            var parentCheckboxes = [];
            data.forEach(function (id) {
                var $checkbox = _this.$treeContainer.find('input[name="' + id + '"]');
                $checkbox.attr('checked', 'checked');
                var $parentItem = _this.getParentItemFromInner($checkbox);
                var $parentCheckbox = $parentItem.find('[data-tree-role="item-label"]').first().find('[data-tree-role="container-checkbox"]');
                if ($.inArray($parentItem.attr('id'), parentIds) === -1) {
                    parentIds.push($parentItem.attr('id'));
                    parentCheckboxes.push($parentCheckbox);
                }
            });
            parentCheckboxes.forEach(function ($parentCheckbox) {
                _this.updateContainerCheckbox($parentCheckbox);
            });
            this.updateRootCheckbox();
        };
        TreeManager.prototype.updateRootCheckbox = function () {
            var $containerCheckboxes = this.$treeContainer.find('[data-tree-role="container-checkbox"]');
            this.disableContainerCheckboxIfNeeded($containerCheckboxes.first());
        };
        TreeManager.prototype.disableContainerCheckboxIfNeeded = function ($checkbox) {
            // Root node: if all children are disabled, then disable it
            if ($checkbox.length > 0) {
                var allDisabled = $checkbox.parents('[data-tree-role="item"]').first()
                    .find('[data-tree-role="children"]')
                    .find('input[type="checkbox"][data-tree-role][data-tree-role!=""]').length == 0;
                if (allDisabled) {
                    $checkbox.attr('disabled', 'disabled');
                }
                else {
                    $checkbox.removeAttr('disabled');
                }
            }
        };
        TreeManager.prototype.scrollTopLeft = function () {
            this.$treeContainer.scrollTop(0);
            this.$treeContainer.scrollLeft(0);
        };
        return TreeManager;
    }());
    exports.TreeManager = TreeManager;
});
